@@ -17,16 +17,18 @@ from django.template import RequestContext, Context
 
 from django.shortcuts import render_to_response
 
-from django.utils.translation import ugettext, ugettext_lazy as _
+from django.utils.translation import ugettext, ungettext, ugettext_lazy as _
 from django.utils.formats import date_format
 
+from django.views.decorators.clickjacking import xframe_options_sameorigin
+
 from sorl.thumbnail.admin import AdminImageMixin
 
 from .models import (
     EmailTemplate, Newsletter, Subscription, Article, Message, Submission
 )
 
-from .utils import now
+from django.utils.timezone import now
 
 from .admin_forms import *
 from .admin_utils import *
@@ -237,6 +239,7 @@ class MessageAdmin(admin.ModelAdmin, ExtendibleModelAdminMixin):
             RequestContext(request, {}),
         )
 
+    @xframe_options_sameorigin
     def preview_html(self, request, object_id):
         message = self._getobj(request, object_id)
 
@@ -258,6 +261,7 @@ class MessageAdmin(admin.ModelAdmin, ExtendibleModelAdminMixin):
 
         return HttpResponse(html_template.render(c))
 
+    @xframe_options_sameorigin
     def preview_text(self, request, object_id):
         message = self._getobj(request, object_id)
 
@@ -271,8 +275,7 @@ class MessageAdmin(admin.ModelAdmin, ExtendibleModelAdminMixin):
             'date': now(),
             'STATIC_URL': settings.STATIC_URL,
             'MEDIA_URL': settings.MEDIA_URL
-            }, autoescape=False
-        )
+        }, autoescape=False)
 
         return HttpResponse(text_template.render(c), mimetype='text/plain')
 
@@ -341,6 +344,8 @@ class SubscriptionAdmin(admin.ModelAdmin, ExtendibleModelAdminMixin):
         'ip', 'subscribe_date', 'unsubscribe_date', 'activation_code'
     )
     date_hierarchy = 'subscribe_date'
+    actions = ['make_subscribed', 'make_unsubscribed']
+
 
     """ List extensions """
     def admin_newsletter(self, obj):
@@ -388,6 +393,23 @@ class SubscriptionAdmin(admin.ModelAdmin, ExtendibleModelAdminMixin):
             return ''
     admin_unsubscribe_date.short_description = _("unsubscribe date")
 
+    """ Actions """
+    def make_subscribed(self, request, queryset):
+        rows_updated = queryset.update(subscribed=True)
+        self.message_user(request, ungettext(
+                "%s user has been successfully subscribed.",
+                "%s users have been successfully subscribed.",
+                rows_updated) % rows_updated)
+    make_subscribed.short_description = _("Subscribe selected users")
+
+    def make_unsubscribed(self, request, queryset):
+        rows_updated = queryset.update(subscribed=False)
+        self.message_user(request, ungettext(
+                "%s user has been successfully unsubscribed.",
+                "%s users have been successfully unsubscribed.",
+                rows_updated) % rows_updated)
+    make_unsubscribed.short_description = _("Unsubscribe selected users")
+
     """ Views """
     def subscribers_import(self, request):
         if request.POST:
