@@ -3,7 +3,8 @@ from django.http import Http404
 from django.utils.functional import update_wrapper
 from django.utils.translation import ugettext_lazy as _
 
-from django.contrib.admin.util import unquote, force_unicode
+from django.contrib.admin.util import unquote
+from django.utils.encoding import force_unicode
 
 
 class ExtendibleModelAdminMixin(object):
