@@ -1,17 +1,25 @@
+# Python 2.5 compatibility
+# Get the with statement from the future
+from __future__ import with_statement
+
 from datetime import timedelta
 
 import time
 
-from django.core import mail
+from django import VERSION as DJANGO_VERSION
 
+from django.core import mail
 from django.core.urlresolvers import reverse
 
+from django.utils.timezone import now
+
+from django.test.utils import override_settings
+
 from ..models import (
     Newsletter, Subscription, Submission, Message,
     EmailTemplate, get_default_sites
 )
 from ..forms import SubscribeRequestForm, UpdateForm, UpdateRequestForm
-from ..utils import now
 
 from .utils import MailTestCase, UserTestCase, WebTestCase, ComparingTestCase
 
@@ -20,23 +28,6 @@ from .utils import MailTestCase, UserTestCase, WebTestCase, ComparingTestCase
 WAIT_TIME = 1
 
 
-class NoNewsLetterListTestCase(WebTestCase):
-    """ Test case for when no newsletter exists """
-
-    def setUp(self):
-        Newsletter.objects.all().delete()
-
-        self.list_url = reverse('newsletter_list')
-
-    def test_list(self):
-        """ Test whether all newsletters are in the list and whether the links
-            to them are correct. """
-
-        response = self.client.get(self.list_url)
-
-        self.assertEqual(response.status_code, 404)
-
-
 class NewsletterListTestCase(WebTestCase):
     """ Base class for newsletter test cases. """
 
@@ -51,6 +42,16 @@ class NewsletterListTestCase(WebTestCase):
 class AnonymousNewsletterListTestCase(NewsletterListTestCase):
     """ Test case for anonymous views of newsletter. """
 
+    def test_emptylist(self):
+        """ No newsletters should yield an emtpy list. """
+
+        # Delete existing newsletters
+        Newsletter.objects.all().delete()
+
+        # Assert a 404 is returned
+        response = self.client.get(self.list_url)
+        self.assertEqual(response.status_code, 404)
+
     def test_list(self):
         """
         Test whether all newsletters are in the list and whether the links
@@ -135,6 +136,23 @@ class AnonymousNewsletterListTestCase(NewsletterListTestCase):
             response = self.client.get(archive_url)
             self.assertContains(response, n.title, status_code=200)
 
+    def test_detail_invisible_not_found(self):
+        """
+        Test whether an invisible newsletter causes a 404 in detail view.
+        """
+
+        # Get an invisible newsletter
+        n = Newsletter.objects.filter(visible=False)[0]
+
+        detail_url = reverse(
+            'newsletter_detail',
+            kwargs={'newsletter_slug': n.slug}
+        )
+
+        response = self.client.get(detail_url)
+
+        self.assertEquals(response.status_code, 404)
+
 
 class UserNewsletterListTestCase(UserTestCase,
                                  NewsletterListTestCase):
@@ -149,8 +167,8 @@ class UserNewsletterListTestCase(UserTestCase,
 
         return subscriptions[0]
 
-    def test_form(self):
-        """ Test whether form elements are present. """
+    def test_listform(self):
+        """ Test whether form elements are present in list. """
 
         response = self.client.get(self.list_url)
 
@@ -161,48 +179,122 @@ class UserNewsletterListTestCase(UserTestCase,
             total_forms, len(self.newsletters.filter(visible=True))
         )
 
-        self.assertContains(
-            response,
-            '<input type="hidden" name="form-TOTAL_FORMS" value="%d" '
-            'id="id_form-TOTAL_FORMS" />' % total_forms
-        )
+        if DJANGO_VERSION[:2] == (1, 4):
+            # Django 1.4
+            self.assertContains(
+                response,
+                '<input type="hidden" name="form-TOTAL_FORMS" value="%d" '
+                'id="id_form-TOTAL_FORMS" />' % total_forms
+            )
 
-        self.assertContains(
-            response,
-            '<input type="hidden" name="form-INITIAL_FORMS" value="%d" '
-            'id="id_form-INITIAL_FORMS" />' % total_forms
-        )
+            self.assertContains(
+                response,
+                '<input type="hidden" name="form-INITIAL_FORMS" value="%d" '
+                'id="id_form-INITIAL_FORMS" />' % total_forms
+            )
+
+        else:
+            # Django 1.5
+            self.assertContains(
+                response,
+                '<input id="id_form-TOTAL_FORMS" name="form-TOTAL_FORMS" '
+                'type="hidden" value="%d" />' % total_forms
+            )
+
+            self.assertContains(
+                response,
+                '<input id="id_form-INITIAL_FORMS" name="form-INITIAL_FORMS" '
+                'type="hidden" value="%d" />' % total_forms
+            )
 
         for form in formset.forms:
-            self.assert_(form.instance.newsletter in self.newsletters,
-                "%s not in %s" % (form.instance.newsletter, self.newsletters))
+            self.assert_(
+                form.instance.newsletter in self.newsletters,
+                "%s not in %s" % (form.instance.newsletter, self.newsletters)
+            )
             self.assertContains(response, form['id'])
             self.assertContains(response, form['subscribed'])
 
-    # TODO
-    # def test_update(self):
-    #     for n in self.newsletters.filter(visible=True):
-    #         response = self.client.post(self.list_url, {'form-0-id': n.id,
-    #                                              'form-0-subscribed': ''})
-    #
-    #         self.assert_(n.subscribed)
-    #
-    # def test_update(self):
-    #     response = self.client.get(self.list_url)
-    #
-    #     formset = response.context['formset']
-    #
-    #     total_forms = self.newsletters.count()
-    #     params = {'form-TOTAL_FORMS' : total_forms,
-    #               'form-INITIAL_FORMS' : total_forms}
-    #
-    #     for n in self.newsletters:
-    #         for x in xrange(0, total_forms):
-    #             field = 'form-%d-id' % x
-    #             params.update({field: self.newsletters[x-1]})
-    #             if n == self.newsletters[x-1]:
-    #                 params.update('form-%d-subscribed' % x: 'checked'})
-    #         response = self.client.post(self.list_url, params)
+    def test_update(self):
+        """ Attempt to subscribe a user to newsletters. """
+
+        # Make sure no subscriptions exist on beforehand
+        Subscription.objects.all().delete()
+
+        # Construct management form data
+        total_forms = self.newsletters.filter(visible=True).count()
+
+        params = {
+            'form-TOTAL_FORMS': total_forms,
+            'form-INITIAL_FORMS': total_forms
+        }
+
+        # Add subscribe to all newsletters
+        count = 0
+        for n in self.newsletters.filter(visible=True):
+            params.update({
+                'form-%d-id' % count: n.id,
+                'form-%d-subscribed' % count: '1'
+            })
+
+            count += 1
+
+        # Post the form
+        response = self.client.post(self.list_url, params)
+
+        # Make sure the result is a success
+        self.assertEquals(response.status_code, 200)
+
+        subscriptions = Subscription.objects.filter(
+            user=self.user
+        )
+
+        # Assert all newsletters have related subscriptions now
+        self.assertTrue(subscriptions.count())
+        self.assertEquals(
+            subscriptions.count(),
+            self.newsletters.filter(visible=True).count()
+        )
+
+    def test_invalid_update(self):
+        """ Test an invalid update, which should fail. """
+        # Make sure no subscriptions exist on beforehand
+        Subscription.objects.all().delete()
+
+        # TODO: Use a Mock to assert a warning has been logged
+        # Ref: http://www.michaelpollmeier.com/python-mock-how-to-assert-a-substring-of-logger-output/
+
+        # A post without any form elements should fail, horribly
+        self.client.post(self.list_url)
+
+        # A post with correct management data with weird values
+        # should cause the formset not to validate.
+
+        # Construct management form data
+        total_forms = self.newsletters.filter(visible=True).count()
+
+        params = {
+            'form-TOTAL_FORMS': total_forms,
+            'form-INITIAL_FORMS': total_forms
+        }
+
+        # Add subscribe to all newsletters
+        count = 0
+        for n in self.newsletters.filter(visible=True):
+            params.update({
+                # Use a wrong value here
+                'form-%d-id' % count: 1000,
+                'form-%d-subscribed' % count: '1'
+            })
+
+            count += 1
+
+        # Post the form
+        self.client.post(self.list_url, params)
+
+        # Assert no subscriptions have been created
+        self.assertFalse(
+            Subscription.objects.filter(subscribed=True).exists())
 
 
 class WebSubscribeTestCase(WebTestCase, MailTestCase):
@@ -271,8 +363,10 @@ class WebUserSubscribeTestCase(WebSubscribeTestCase,
         self.assertEqual(response.context['newsletter'], self.n)
         self.assertEqual(response.context['user'], self.user)
 
-        self.assertContains(response,
-            'action="%s"' % self.subscribe_confirm_url)
+        self.assertContains(
+            response,
+            'action="%s"' % self.subscribe_confirm_url
+        )
         self.assertContains(response, 'id="id_submit"')
 
         subscription = self.get_user_subscription()
@@ -336,6 +430,18 @@ class WebUserSubscribeTestCase(WebSubscribeTestCase,
         self.assert_(subscription.subscribed)
         self.assertFalse(subscription.unsubscribed)
 
+    def test_unsubscribe_not_subscribed_view(self):
+        """
+        Test attempting to unsubscriped a user without a subscription.
+        """
+
+        response = self.client.get(self.unsubscribe_url)
+
+        self.assertIn(
+            'You are not subscribed to',
+            unicode(list(response.context['messages'])[0])
+        )
+
     def test_unsubscribe_post(self):
         """ Test unsubscription confirmation. """
         subscription = Subscription(user=self.user, newsletter=self.n)
@@ -371,8 +477,10 @@ class WebUserSubscribeTestCase(WebSubscribeTestCase,
         self.assertEqual(response.context['newsletter'], self.n)
         self.assertEqual(response.context['user'], self.user)
 
-        self.assertNotContains(response,
-            'action="%s"' % self.unsubscribe_confirm_url)
+        self.assertNotContains(
+            response,
+            'action="%s"' % self.unsubscribe_confirm_url
+        )
         self.assertNotContains(response, 'id="id_submit"')
 
 
@@ -382,13 +490,12 @@ class AnonymousSubscribeTestCase(WebSubscribeTestCase,
     def test_subscribe_request_view(self):
         """ Test the subscription form. """
 
-        response = self.client.get(self.subscribe_url)
+        response = self.app.get(self.subscribe_url)
 
         self.assertContains(response, self.n.title, status_code=200)
-        self.assertContains(response,
-            'input id="id_name_field" type="text" name="name_field"')
-        self.assertContains(response,
-            'input id="id_email_field" type="text" name="email_field"')
+
+        self.assertIn('name_field', response.form.fields)
+        self.assertIn('email_field', response.form.fields)
 
         self.assertEqual(response.context['newsletter'], self.n)
 
@@ -412,6 +519,7 @@ class AnonymousSubscribeTestCase(WebSubscribeTestCase,
 
         self.assertInContext(response, 'newsletter', Newsletter, self.n)
         self.assertInContext(response, 'form', SubscribeRequestForm)
+        self.assertFalse(response.context['error'])
 
         subscription = getattr(response.context['form'], 'instance', None)
         self.assert_(subscription)
@@ -426,6 +534,27 @@ class AnonymousSubscribeTestCase(WebSubscribeTestCase,
 
         self.assertEmailContains(full_activate_url)
 
+    # Only run this test when settings overrides are available
+    def test_subscrube_request_post_error(self):
+        """
+        Test whether a failing subscribe request email generated an error in
+        the context.
+
+        We do this by overriding the default mail backend to a settings which
+        we know pretty sure is bound to fail.
+        """
+
+        with override_settings(EMAIL_BACKEND='django.core.mail.backends.smtp.EmailBackend'):
+            with override_settings(EMAIL_PORT=12345678):
+                response = self.client.post(
+                    self.subscribe_url, {
+                        'name_field': 'Test Name',
+                        'email_field': 'test@ifjoidjsufhdsidhsuufihs.dfs'
+                    }
+                )
+
+        self.assertTrue(response.context['error'])
+
     def test_retry_subscribe(self):
         """
         When requesting a subscription for an e-mail address for which
@@ -493,11 +622,13 @@ class AnonymousSubscribeTestCase(WebSubscribeTestCase,
 
         # Unsubscribe
         response = self.client.post(
-            subscription.unsubscribe_activate_url(), {
+            subscription.unsubscribe_activate_url(),
+            {
                 'name_field': subscription.name,
                 'email_field': subscription.email,
                 'user_activation_code': subscription.activation_code
-        })
+            }
+        )
 
         self.assertEquals(response.status_code, 200)
 
@@ -508,10 +639,12 @@ class AnonymousSubscribeTestCase(WebSubscribeTestCase,
 
         # Resubscribe request
         response = self.client.post(
-            self.subscribe_url, {
+            self.subscribe_url,
+            {
                 'name_field': subscription.name,
                 'email_field': subscription.email,
-        })
+            }
+        )
 
         self.assertEquals(response.status_code, 200)
 
@@ -520,11 +653,13 @@ class AnonymousSubscribeTestCase(WebSubscribeTestCase,
 
         # Activate subscription
         response = self.client.post(
-            subscription.subscribe_activate_url(), {
+            subscription.subscribe_activate_url(),
+            {
                 'name_field': subscription.name,
                 'email_field': subscription.email,
                 'user_activation_code': subscription.activation_code
-        })
+            }
+        )
         self.assertInContext(response, 'form', UpdateForm)
 
         subscription = getattr(response.context['form'], 'instance', None)
@@ -552,9 +687,11 @@ class AnonymousSubscribeTestCase(WebSubscribeTestCase,
                 }
             )
 
-            self.assertContains(response,
+            self.assertContains(
+                response,
                 "Please log in as that user and try again.",
-                status_code=200)
+                status_code=200
+            )
 
     def test_subscribe_request_activate(self):
         """ Test subscription activation. """
@@ -612,6 +749,7 @@ class AnonymousSubscribeTestCase(WebSubscribeTestCase,
 
         self.assertInContext(response, 'newsletter', Newsletter, self.n)
         self.assertInContext(response, 'form', UpdateRequestForm)
+        self.assertFalse(response.context['error'])
 
         self.assertEqual(subscription, response.context['form'].instance)
 
@@ -623,13 +761,35 @@ class AnonymousSubscribeTestCase(WebSubscribeTestCase,
 
         self.assertEmailContains(full_activate_url)
 
+    def test_unsubscribe_request_post_error(self):
+        """
+        Test whether a failing unsubscribe request email generated an error in
+        the context.
+
+        We do this by overriding the default mail backend to a settings which
+        we know pretty sure is bound to fail.
+        """
+        subscription = Subscription(newsletter=self.n,
+                                    name='Test Name',
+                                    email='test@email.com',
+                                    subscribed=True)
+        subscription.save()
+
+        with override_settings(EMAIL_BACKEND='django.core.mail.backends.smtp.EmailBackend'):
+            with override_settings(EMAIL_PORT=12345678):
+                response = self.client.post(
+                    self.unsubscribe_url, {'email_field': 'test@email.com'}
+                )
+
+        self.assertTrue(response.context['error'])
+
     def test_unsubscribe_request_view(self):
         """ Test the unsubscribe request form. """
-        response = self.client.get(self.unsubscribe_url)
+        response = self.app.get(self.unsubscribe_url)
 
         self.assertContains(response, self.n.title, status_code=200)
-        self.assertContains(response,
-            'input id="id_email_field" type="text" name="email_field"')
+
+        self.assertIn('email_field', response.form.fields)
 
         self.assertEqual(response.context['newsletter'], self.n)
 
@@ -668,11 +828,10 @@ class AnonymousSubscribeTestCase(WebSubscribeTestCase,
     def test_update_request_view(self):
         """ Test the update request form. """
 
-        response = self.client.get(self.update_url)
+        response = self.app.get(self.update_url)
 
         self.assertContains(response, self.n.title, status_code=200)
-        self.assertContains(response,
-            'input id="id_email_field" type="text" name="email_field"')
+        self.assertIn('email_field', response.form.fields)
 
         self.assertEqual(response.context['newsletter'], self.n)
 
@@ -696,6 +855,7 @@ class AnonymousSubscribeTestCase(WebSubscribeTestCase,
 
         self.assertInContext(response, 'newsletter', Newsletter, self.n)
         self.assertInContext(response, 'form', UpdateRequestForm)
+        self.assertFalse(response.context['error'])
 
         self.assertEqual(subscription, response.context['form'].instance)
 
@@ -707,6 +867,28 @@ class AnonymousSubscribeTestCase(WebSubscribeTestCase,
 
         self.assertEmailContains(full_activate_url)
 
+    def test_update_request_post_error(self):
+        """
+        Test whether a failing update request email generated an error in
+        the context.
+
+        We do this by overriding the default mail backend to a settings which
+        we know pretty sure is bound to fail.
+        """
+        subscription = Subscription(newsletter=self.n,
+                                    name='Test Name',
+                                    email='test@email.com',
+                                    subscribed=True)
+        subscription.save()
+
+        with override_settings(EMAIL_BACKEND='django.core.mail.backends.smtp.EmailBackend'):
+            with override_settings(EMAIL_PORT=12345678):
+                response = self.client.post(
+                    self.update_url, {'email_field': 'test@email.com'}
+                )
+
+        self.assertTrue(response.context['error'])
+
     def test_unsubscribe_update_unactivated(self):
         """ Test updating unsubscribed subscriptions view. """
 
@@ -765,6 +947,28 @@ class AnonymousSubscribeTestCase(WebSubscribeTestCase,
         self.assertEqual(subscription.name, testname2)
         self.assertEqual(subscription.email, testemail2)
 
+    def test_update_request_activate_form(self):
+        """
+        Test requesting a form for activating an update without activation
+        code in the URL.
+        """
+
+        subscription = Subscription(newsletter=self.n,
+                                    name='Test Name',
+                                    email='test@email.com')
+        subscription.save()
+
+        activate_url = reverse('newsletter_update', kwargs={
+            'newsletter_slug': self.n.slug,
+            'action': 'update',
+            'email': subscription.email
+        })
+
+        response = self.client.get(activate_url)
+
+        # Make sure the form is there
+        self.assertInContext(response, 'form', UpdateForm)
+
 
 class ArchiveTestcase(NewsletterListTestCase):
     def setUp(self):
@@ -831,6 +1035,18 @@ class ArchiveTestcase(NewsletterListTestCase):
 
         self.assertContains(response, self.submission.message.title)
 
+    def test_archive_notemplate(self):
+        """
+        The archive detail should return 404 when the template has no HTML. """
+
+        # Make sure the HTML is deleted
+        EmailTemplate.objects.update(html=None)
+
+        detail_url = self.submission.get_absolute_url()
+
+        response = self.client.get(detail_url)
+        self.assertEqual(response.status_code, 404)
+
     def test_archive_unpublished_detail(self):
         """ Assert that an unpublished submission is truly inaccessible. """
 
